﻿/*
 * TCPEchoServer
 *
 * Author Michael Claudius, ZIBAT Computer Science
 * Version 1.0. 2014.02.10
 * Copyright 2014 by Michael Claudius
 * Revised 2014.09.01, 2015.10.16; 2016.11.10
 * All rights reserved
 */


using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Net.Sockets;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Threading.Tasks;

// Adapted from https://msdn.microsoft.com/en-us/library/system.net.security.sslstream.aspx

namespace TCPEchoServer
{
    class TCPEchoServer
    {

        public static void Main(string[] args)
        {

            string serverCertificateFile = "c:/certificates/ServerSSL.cer";  // or ServerSSL.pfx using mysecret
            bool clientCertificateRequired = false;
            bool checkCertificateRevocation = false;
            SslProtocols enabledSSLProtocols = SslProtocols.Tls;  //superseeds the former SslProtocols.Ssl3

            X509Certificate serverCertificate = new X509Certificate(serverCertificateFile, "secret");  //mysecret
            //Alternatively
            // X509Certificate ServerCertificate = X509Certificate.CreateFromCertFile(ServerCertificateFile);  

            TcpListener serverSocket = new TcpListener(6789);

            serverSocket.Start();
            Console.WriteLine("Server started");

            TcpClient connectionSocket = serverSocket.AcceptTcpClient();
            Console.WriteLine("Server activated");

            Stream unsecureStream = connectionSocket.GetStream();
            
            bool leaveInnerStreamOpen = false; //i.e. close the unsecureStream when sslStream is closed
            // Decorator/Wrapper design pattern
            SslStream sslStream = new SslStream(unsecureStream, leaveInnerStreamOpen);
           sslStream.AuthenticateAsServer(serverCertificate, clientCertificateRequired, enabledSSLProtocols, checkCertificateRevocation);

            //Alternatively just
            //sslStream.AuthenticateAsServer(serverCertificate);


           Console.WriteLine("Server authenticated");

            StreamReader sr = new StreamReader(sslStream);
            StreamWriter sw = new StreamWriter(sslStream);
            sw.AutoFlush = true; // enable automatic flushing

            string message = sr.ReadLine();
            string answer = "";
            while (message != null && message != "")
            {
                Console.WriteLine("Client: " + message);
                answer = message.ToUpper();
                sw.WriteLine(answer);
                message = sr.ReadLine();

            }

            sslStream.Close();
            connectionSocket.Close();
            serverSocket.Stop();

        }

        //Not used
        private static bool ValidateClientCertificate(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
        {
            if (sslPolicyErrors == SslPolicyErrors.None)
            {
                Console.WriteLine("Certificate validation successful.");
                return true;
            }
            Console.WriteLine("Errors in certificate validation:");
            Console.WriteLine(sslPolicyErrors);
            return false;
        }

    }
    
}
